/* $Id: HymnController.m,v 1.1 2004/05/10 00:16:59 playfair Exp $
**
** This program is free software; you can redistribute it and/or
** modify it under the terms of the GNU General Public License as
** published by the Free Software Foundation; either version 2 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful, but
** WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
** General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
** 02111-1307, USA.
*/

#import "HymnController.h"
#import "config.h"

@implementation HymnController

/* From NSObject */
- (id)init
{
    if (self == [super init]) {
        infiles = [[NSMutableSet alloc] init];
    }
    return self;
}

- (void)dealloc
{
    [infiles release];
    [super dealloc];
}

/* NSApplication delegate methods */
- (BOOL)applicationShouldTerminateAfterLastWindowClosed:(NSApplication*)theApplication
{
    return YES;
}

- (IBAction)orderFrontStandardAboutPanel:(id)sender
{
    /* this method is overridden so we can pull the version info from config.h */
    NSString *version = [NSString stringWithCString:VERSION];
    NSMutableDictionary *opts = [NSMutableDictionary dictionaryWithCapacity:1];
    [opts setObject:version forKey:@"Version"];
    [[NSApplication sharedApplication] orderFrontStandardAboutPanelWithOptions:opts];
}

/* HymnController methods */
- (IBAction)clear:(id)sender
{
    [[[fileList textStorage] mutableString] setString:@""];
    [convertButton setEnabled:NO];
    [clearButton setEnabled:NO];
    [progress setDoubleValue:0];
    [progress display];
    [self writeStatus:@"Idle.  Drag m4p files here ..."];
}

- (IBAction)convert:(id)sender
{
	hymn_callback_t hcb;
	int err;
	unsigned int i, count;
	double percent;
	BOOL shouldCopyTags, shouldCopyArtwork;
	
	hcb.errmsg = writeError;
	hcb.logmsg = writeLog;
	hcb.status = writeStatus;
    hcb.log_lvl = 4;
	hcb.user_data = self;

    NSArray *sortedFiles = [[infiles allObjects] sortedArrayUsingSelector:@selector(compare:)];
	count = [sortedFiles count];
	percent = 0;
	for (i=0; i<count; ++i) {
		NSString *infile = [sortedFiles objectAtIndex:i];
		/* TODO: rewrite this to do proper file extension replacement */
		/*       allow user to specify output directory */
		NSString *outfile = [[infile substringToIndex:[infile length] - 3] stringByAppendingString:@"m4a"];
        err = convert(&hcb, [infile cString], [outfile cString]);

		percent = ((double)(i+1)) / ((double)count) * 100.0;
		[progress setDoubleValue:percent];
		[progress display];
	
		if (err) break;
	}
	
	if (err == ERR_SUCCESS) {
		[self writeStatus:@"Done.  Drag m4p files here ..."];
	}

    [progress setDoubleValue:100];
    [progress display];
	[convertButton setEnabled:NO];
}

- (void)setFiles:(NSArray*)files
{
    if (![convertButton isEnabled]) {
        [infiles removeAllObjects];
    }
    [infiles addObjectsFromArray: files];
    
    NSArray *sortedFiles = [[infiles allObjects] sortedArrayUsingSelector:@selector(compare:)];
    NSString *content = [sortedFiles componentsJoinedByString:@"\n"];
    [[[fileList textStorage] mutableString] setString:content];
    
    [convertButton setEnabled:YES];
    [clearButton setEnabled:YES];
    [self writeStatus:@"Click Convert to convert files ..."];
}

- (IBAction)showLogMessages:(id)sender
{
    [logPanel makeKeyAndOrderFront:sender];
}

- (void)writeError:(NSString*)err
{
	[status setTextColor:[NSColor redColor]];
	[status setStringValue:err];
	[status display];
}

- (void)writeLog:(NSString*)str
{
    [[[logMessages textStorage] mutableString] appendString:str];
}

- (void)writeStatus:(NSString*)str
{
	[status setTextColor:[NSColor blackColor]];
	[status setStringValue:str];
	[status display];
}
@end

int writeError(id self, int err, ...)
{
    char errMsg[1024];

	va_list argp;
    va_start(argp, err);
    sprintf(errMsg, ERROR_MESSAGES[err], argp);
    va_end(argp);
    [self writeError:[NSString stringWithCString:errMsg]];
	
    return err;
}

void writeLog(id self, int log_lvl, int msg_lvl, char *str, ...)
{
    if (log_lvl >= msg_lvl) {
        char msg[1024];
 
        sprintf(msg, "hymn(%d): ", msg_lvl);
        [self writeLog:[NSString stringWithCString:msg]];

        va_list argp;
        va_start(argp, str);
        vsprintf(msg, str, argp);
        va_end(argp);

        [self writeLog:[NSString stringWithCString:msg]];
        [self writeLog:@"\n"];
    }
}

void writeStatus(id self, char *status)
{
    [self writeStatus:[NSString stringWithCString:status]];
}
