/*****************************************************************************
* $Id: hymn.c,v 1.3 2004/05/10 01:13:10 playfair Exp $
 *****************************************************************************
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111,
 * USA.
 *****************************************************************************/

#include <fcntl.h>
#include <stdbool.h>
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/stat.h>

#include "config.h"

#include "hymn.h"
#include "hymn_private.h"

#include "endianutils.h"
#include "atoms.h"

#include "aes.h"
#include "md5.h"

/* functions */
int hymn_init(hymn_callback_t *hcb, hymn_t *ctx,
              const char *infilename, const char *outfile) {
    FILE *infile;

#ifdef WIN32
    _fmode = _O_BINARY;
#endif
    ctx->num_sample_sizes = 0;
    ctx->num_sample_offsets = 0;
    ctx->sinf_size = 0;
    ctx->write_off = false;
    ctx->iter = 0;
    infile = fopen(infilename, "r");
    if (infile) {
        struct stat instat;
        int statok = stat(infilename, &instat);
        if (!statok) {
            ctx->inbuf_size = instat.st_size;
            ctx->inbuf = malloc(ctx->inbuf_size);
            if (ctx->inbuf) {
                int bytes_read = fread(ctx->inbuf, 1, ctx->inbuf_size, infile);
                if (bytes_read != ctx->inbuf_size) {
                    return hcb->errmsg(hcb->user_data, ERR_BYTESREAD,
                                       ctx->inbuf_size, bytes_read);
                }
                fclose(infile);
            } else {
                return hcb->errmsg(hcb->user_data, ERR_BYTESALLOC,
                                   ctx->inbuf_size);
            }
        } else {
            return hcb->errmsg(hcb->user_data, ERR_FILESTAT, infile);
        }
    } else {
        return hcb->errmsg(hcb->user_data, ERR_FILEREAD, infile);
    }

    ctx->outfile = fopen(outfile, "w+");
    if (!ctx->outfile) {
        return hcb->errmsg(hcb->user_data, ERR_FILEWRITE, outfile);
    }

    return SUCCESS;
}

int hymn_free(hymn_callback_t *hcb, hymn_t *ctx) {
    if (ctx->inbuf) free(ctx->inbuf);
    if (ctx->drms_iviv) free(ctx->drms_iviv);
    if (ctx->drms_name) free(ctx->drms_name);
    if (ctx->drms_priv) free(ctx->drms_priv);
    if (ctx->sample_table_sizes) free(ctx->sample_table_sizes);
    if (ctx->sample_table_offsets) free(ctx->sample_table_offsets);
    fclose(ctx->outfile);
}

int remux(hymn_callback_t *hcb, char *infile, char *outfile) {
    int ret;
    hymn_t ctx;
    if ( (ret = hymn_init(hcb, &ctx, infile, outfile) ) ) {
        return ret;
    } else {
        /* We will iterate over our input file two times.  The
         * first time we are parsing.  The second time we are
         * writing out our output file after having collected all
         * of the necessary information from the input file.
         */
        for (ctx.iter=0; ctx.iter<2; ++ctx.iter) {
            uint8_t *fp = ctx.inbuf;
            if ( (ret = parse_atoms(hcb, &ctx, &fp, ctx.inbuf_size, 0) ) ) {
                return ret;
            }
        }
        hymn_free(hcb, &ctx);

        return SUCCESS;
    }
}

int convert(hymn_callback_t *hcb, char *infile, char *outfile) {
    int err;

    if ( ( err = remux(hcb, infile, outfile) ) ) {
        return err;
    }

    return SUCCESS;
}

/* EOF */
