#ifndef __HYMN_H__
#define __HYMN_H__
/*****************************************************************************
* $Id: hymn.h,v 1.2 2004/05/10 01:13:10 playfair Exp $
*****************************************************************************
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License as
* published by the Free Software Foundation; either version 2 of the
* License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
*n Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111,
* USA.
*****************************************************************************/

/* file callback structure */
typedef struct
{
    int (*errmsg)(void *user_data, int err, ...);
    void (*logmsg)(void *user_data, int log_lvl, int msg_lvl, char *str, ...);
    void (*status)(void *user_data, char *status);
    void *user_data;
    int log_lvl;
} hymn_callback_t;

/* interface */
int convert(hymn_callback_t *hcb, char *infile, char *outfile);

/* log levels */
static int LOG_OFF = 0;
static int LOG_WARN = 1;
static int LOG_INFO = 2;
static int LOG_DEBUG = 3;
static int LOG_TRACE = 4;
static int LOG_SPEW = 5;

/* error codes */
static int SUCCESS = 0;
static int ERR_FILEREAD = 1;
static int ERR_FILESTAT = 2;
static int ERR_BYTESALLOC = 3;
static int ERR_BYTESREAD = 4;
static int ERR_FILEWRITE = 5;
static int ERR_NOAACTRACK = 6;
static int ERR_REMUX = 7;
static int ERR_METADATA = 8;
static int ERR_NOKEY = 9;
static int ERR_GET_USER_KEY = 10;
static int ERR_KEY_DECRYPT = 11;

/* error message array -- must keep in sync w/ hymn.h */
static const char *ERROR_MESSAGES[12] =
{
    /*SUCCESS*/                    "Success.",
    /*ERR_FILEREAD*/               "Couldn't open file: %s",
    /*ERR_FILESTAT*/               "Couldn't determine file size of %s.",
    /*ERR_BYTESALLOC*/             "Couldn't allocate %d bytes.",
    /*ERR_BYTESREAD*/              "Tried to read %d bytes, read only %d.",
    /*ERR_FILEWRITE*/              "Couldn't write file: %s",
    /*ERR_NOAACTRACK*/             "Couldn't find AAC track in input file.",
    /*ERR_REMUX*/                  "Couldn't decode input file.",
    /*ERR_METADATA*/               "Couldn't set meta data on output file.",
    /*ERR_NOKEY*/                  "Couldn't find iTunes key or an iPod.",
    /*ERR_GET_USER_KEY*/           "Couldn't get DRM key for user.",
    /*ERR_KEY_DECRYPT*/            "Couldn't decrypt secret key in file."
};

#endif /* __HYMN_H__ */
